@@ -200,6 +200,7 @@ namespace XrdCl
     REGISTER_VAR_INT( varsInt, "TCPKeepAliveInterval", DefaultTCPKeepAliveInterval );
     REGISTER_VAR_INT( varsInt, "TCPKeepProbes",        DefaultTCPKeepAliveProbes   );
     REGISTER_VAR_INT( varsInt, "MultiProtocol",        DefaultMultiProtocol        );
+    REGISTER_VAR_INT( varsInt, "ParallelEvtLoop",      DefaultParallelEvtLoop      );
 
     REGISTER_VAR_STR( varsStr, "PollerPreference",     DefaultPollerPreference     );
     REGISTER_VAR_STR( varsStr, "ClientMonitor",        DefaultClientMonitor        );
@@ -207,6 +208,8 @@ namespace XrdCl
     REGISTER_VAR_STR( varsStr, "NetworkStack",         DefaultNetworkStack         );
     REGISTER_VAR_STR( varsStr, "PlugIn",               DefaultPlugIn               );
     REGISTER_VAR_STR( varsStr, "PlugInConfDir",        DefaultPlugInConfDir        );
+    REGISTER_VAR_STR( varsStr, "ReadRecovery",         DefaultReadRecovery         );
+    REGISTER_VAR_STR( varsStr, "WriteRecovery",        DefaultWriteRecovery        );
 
     //--------------------------------------------------------------------------
     // Process the configuration files
