@@ -40,6 +40,38 @@
 
 namespace
 {
+  //----------------------------------------------------------------------------
+  //! Wrapper class used to delete FileSystem object
+  //----------------------------------------------------------------------------
+  class DeallocFSHandler: public XrdCl::ResponseHandler
+  {
+    public:
+      //------------------------------------------------------------------------
+      // Constructor and destructor
+      //------------------------------------------------------------------------
+      DeallocFSHandler( XrdCl::FileSystem *fs, ResponseHandler *userHandler ):
+        pFS(fs), pUserHandler(userHandler) {}
+
+      virtual ~DeallocFSHandler()
+      {
+        delete pFS;
+      }
+
+      //------------------------------------------------------------------------
+      // Handle the response
+      //------------------------------------------------------------------------
+      virtual void HandleResponse( XrdCl::XRootDStatus *status,
+                                   XrdCl::AnyObject    *response )
+      {
+        pUserHandler->HandleResponse(status, response);
+        delete this;
+      }
+
+    private:
+      XrdCl::FileSystem      *pFS;
+      ResponseHandler *pUserHandler;
+  };
+
   //----------------------------------------------------------------------------
   // Deep locate handler
   //----------------------------------------------------------------------------
@@ -151,8 +183,8 @@ namespace
           //--------------------------------------------------------------------
           if( it->IsManager() )
           {
-            FileSystem fs( it->GetAddress() );
-            if( fs.Locate( pPath, pFlags, this, pExpires-::time(0)).IsOK() )
+            FileSystem *fs = new FileSystem( it->GetAddress() );
+            if( fs->Locate( pPath, pFlags, new DeallocFSHandler(fs, this), pExpires-::time(0)).IsOK() )
               ++pOutstanding;
           }
         }
