@@ -135,14 +135,23 @@ namespace XrdCl
     XrdSysMutexHelper scopedLock( pMutex );
     int         errNum = 0;
     const char *errMsg = 0;
-    pPoller = IOEvents::Poller::Create( errNum, &errMsg );
-    if( !pPoller )
+
+    for( int i = 0; i < pNbPoller; ++i )
     {
-      log->Error( PollerMsg, "Unable to create the internal poller object: ",
-                             "%s (%s)", strerror( errno ), errMsg );
-      return false;
+      XrdSys::IOEvents::Poller* poller = IOEvents::Poller::Create( errNum, &errMsg );
+      if( !poller )
+      {
+        log->Error( PollerMsg, "Unable to create the internal poller object: ",
+                               "%s (%s)", strerror( errno ), errMsg );
+        return false;
+      }
+      pPollerPool.push_back( poller );
     }
 
+    pNext = pPollerPool.begin();
+
+    log->Debug( PollerMsg, "Using %d poller threads", pNbPoller );
+
     //--------------------------------------------------------------------------
     // Check if we have any descriptors to reinsert from the last time we
     // were started
@@ -152,7 +161,7 @@ namespace XrdCl
     {
       PollerHelper *helper = (PollerHelper*)it->second;
       Socket       *socket = it->first;
-      helper->channel = new IOEvents::Channel( pPoller, socket->GetFD(),
+      helper->channel = new IOEvents::Channel( GetPoller( socket ), socket->GetFD(),
                                                helper->callBack );
       if( helper->readEnabled )
       {
@@ -194,19 +203,25 @@ namespace XrdCl
     log->Debug( PollerMsg, "Stopping the poller..." );
 
     XrdSysMutexHelper scopedLock( pMutex );
-    if( !pPoller )
+
+    if( pPollerPool.empty() )
     {
       log->Debug( PollerMsg, "Stopping a poller that has not been started" );
       return true;
     }
 
-    XrdSys::IOEvents::Poller *poller = pPoller;
-    pPoller = 0;
+    while( !pPollerPool.empty() )
+    {
+      XrdSys::IOEvents::Poller *poller = pPollerPool.back();
+      pPollerPool.pop_back();
 
-    scopedLock.UnLock();
-    poller->Stop();
-    delete poller;
-    scopedLock.Lock( &pMutex );
+      if( !poller ) continue;
+
+      scopedLock.UnLock();
+      poller->Stop();
+      delete poller;
+      scopedLock.Lock( &pMutex );
+    }
 
     SocketMap::iterator  it;
     const char          *errMsg = 0;
@@ -266,12 +281,14 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Create the socket helper
     //--------------------------------------------------------------------------
+    XrdSys::IOEvents::Poller* poller = GetPoller( socket );
+
     PollerHelper *helper = new PollerHelper();
     helper->callBack = new ::SocketCallBack( socket, handler );
 
-    if( pPoller )
+    if( poller )
     {
-      helper->channel  = new XrdSys::IOEvents::Channel( pPoller,
+      helper->channel  = new XrdSys::IOEvents::Channel( poller,
                                                         socket->GetFD(),
                                                         helper->callBack );
     }
@@ -300,11 +317,18 @@ namespace XrdCl
     log->Debug( PollerMsg, "%s Removing socket from the poller",
                            socket->GetName().c_str() );
 
+    // decrement the counter associated with the respective channel ID
+    --(pPollerMap[socket->GetChannelID()].second);
+    // if this was the last socket remove the entry
+    if( pPollerMap[socket->GetChannelID()].second == 0 )
+      pPollerMap.erase(socket->GetChannelID());
+
     //--------------------------------------------------------------------------
     // Remove the socket
     //--------------------------------------------------------------------------
     PollerHelper *helper = (PollerHelper*)it->second;
-    if( pPoller )
+
+    if( helper->channel )
     {
       const char *errMsg;
       bool status = helper->channel->Disable( Channel::allEvents, &errMsg );
@@ -351,6 +375,7 @@ namespace XrdCl
     }
 
     PollerHelper *helper = (PollerHelper*)it->second;
+    XrdSys::IOEvents::Poller* poller = pPollerMap[socket->GetChannelID()].first;
 
     //--------------------------------------------------------------------------
     // Enable read notifications
@@ -364,7 +389,7 @@ namespace XrdCl
       log->Dump( PollerMsg, "%s Enable read notifications, timeout: %d",
                             socket->GetName().c_str(), timeout );
 
-      if( pPoller )
+      if( poller )
       {
         const char *errMsg;
         bool status = helper->channel->Enable( Channel::readEvents, timeout,
@@ -390,7 +415,7 @@ namespace XrdCl
       log->Dump( PollerMsg, "%s Disable read notifications",
                             socket->GetName().c_str() );
 
-      if( pPoller )
+      if( poller )
       {
         const char *errMsg;
         bool status = helper->channel->Disable( Channel::readEvents, &errMsg );
@@ -435,6 +460,7 @@ namespace XrdCl
     }
 
     PollerHelper *helper = (PollerHelper*)it->second;
+    XrdSys::IOEvents::Poller* poller = pPollerMap[socket->GetChannelID()].first;
 
     //--------------------------------------------------------------------------
     // Enable write notifications
@@ -449,7 +475,7 @@ namespace XrdCl
       log->Dump( PollerMsg, "%s Enable write notifications, timeout: %d",
                             socket->GetName().c_str(), timeout );
 
-      if( pPoller )
+      if( poller )
       {
         const char *errMsg;
         bool status = helper->channel->Enable( Channel::writeEvents, timeout,
@@ -474,7 +500,7 @@ namespace XrdCl
 
       log->Dump( PollerMsg, "%s Disable write notifications",
                             socket->GetName().c_str() );
-      if( pPoller )
+      if( poller )
       {
         const char *errMsg;
         bool status = helper->channel->Disable( Channel::writeEvents, &errMsg );
@@ -499,4 +525,44 @@ namespace XrdCl
     SocketMap::iterator it = pSocketMap.find( socket );
     return it != pSocketMap.end();
   }
+
+  //----------------------------------------------------------------------------
+  // Return poller threads in round-robin fashion
+  //----------------------------------------------------------------------------
+  XrdSys::IOEvents::Poller* PollerBuiltIn::GetNextPoller()
+  {
+    PollerPool::iterator ret = pNext;
+    ++pNext;
+    if( pNext == pPollerPool.end() )
+      pNext = pPollerPool.begin();
+    return *ret;
+  }
+
+  //----------------------------------------------------------------------------
+  // Return the poller associated with the respective channel
+  //----------------------------------------------------------------------------
+  XrdSys::IOEvents::Poller* PollerBuiltIn::GetPoller(const Socket * socket)
+  {
+    PollerMap::iterator itr = pPollerMap.find(socket->GetChannelID());
+    if( itr == pPollerMap.end())
+    {
+      XrdSys::IOEvents::Poller* poller = GetNextPoller();
+      pPollerMap[socket->GetChannelID()] = std::make_pair(poller, 1);
+      return poller;
+    }
+
+    ++(itr->second.second);
+    return itr->second.first;
+  }
+
+  //----------------------------------------------------------------------------
+  // Get the initial value for pNbPoller
+  //----------------------------------------------------------------------------
+  int PollerBuiltIn::GetNbPollerInit()
+  {
+    Env * env = DefaultEnv::GetEnv();
+    int ret = XrdCl::DefaultParallelEvtLoop;
+    env->GetInt("ParallelEvtLoop", ret);
+    return ret;
+  }
 }
