@@ -146,6 +146,7 @@ namespace XrdCl
         return Take | RemoveHandler;
 
       case kXR_waitresp:
+        pResponse = 0;
         return Take | Ignore; // This must be handled synchronously!
 
       //------------------------------------------------------------------------
@@ -556,8 +557,10 @@ namespace XrdCl
       }
 
       //------------------------------------------------------------------------
-      // kXR_waitresp - the response will be returned in some seconds
-      // as an unsolicited message
+      // kXR_waitresp - the response will be returned in some seconds as an
+      // unsolicited message. Currently all messages of this type are handled
+      // one step before in the XrdClStream::OnIncoming as they need to be
+      // processed synchronously.
       //------------------------------------------------------------------------
       case kXR_waitresp:
       {
@@ -1740,7 +1743,7 @@ namespace XrdCl
           (status.errNo == kXR_FSError || status.errNo == kXR_IOError ||
           status.errNo == kXR_ServerError || status.errNo == kXR_NotFound) )
       {
-        UpdateTriedCGI();
+        UpdateTriedCGI(status.errNo);
         if( status.errNo == kXR_NotFound )
           SwitchOnRefreshFlag();
         HandleError( RetryAtServer( pLoadBalancer.url ) );
@@ -1812,11 +1815,20 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Update the "tried=" part of the CGI of the current message
   //----------------------------------------------------------------------------
-  void XRootDMsgHandler::UpdateTriedCGI()
+  void XRootDMsgHandler::UpdateTriedCGI(uint32_t errNo)
   {
     URL::ParamsMap cgi;
     std::string    tried = pUrl.GetHostName();
 
+    // Report the reason for the failure to the next location
+    //
+    if (errNo)
+       {     if (errNo == kXR_NotFound)     cgi["triedrc"] = "enoent";
+        else if (errNo == kXR_IOError)      cgi["triedrc"] = "ioerr";
+        else if (errNo == kXR_FSError)      cgi["triedrc"] = "fserr";
+        else if (errNo == kXR_ServerError)  cgi["triedrc"] = "srverr";
+       }
+
     //--------------------------------------------------------------------------
     // If our current load balancer is a metamanager and we failed either
     // at a diskserver or at an unidentified node we also exclude the last
