@@ -78,21 +78,27 @@ class XrdCmsDrop : XrdJob
 {
 public:
 
-     void DoIt() {Cluster.STMutex.Lock();
-                  int rc = Cluster.Drop(nodeEnt, nodeInst, this);
-                  Cluster.STMutex.UnLock();
-                  if (!rc) delete this;
+     void DoIt() {if (nodeP)
+                     {nodeP->Delete(Cluster.STMutex);
+                      delete this;
+                     } else {
+                      if (!Cluster.Drop(nodeEnt, nodeInst, this)) delete this;
+                     }
                  }
 
-          XrdCmsDrop(int nid, int inst) : XrdJob("drop node")
-                    {nodeEnt  = nid;
-                     nodeInst = inst;
-                     Sched->Schedule((XrdJob *)this, time(0)+Config.DRPDelay);
-                    }
+          XrdCmsDrop(XrdCmsNode *nP) : XrdJob("delete node"), nodeP(nP),
+                                       nodeEnt(0), nodeInst(0)
+                    {Sched->Schedule((XrdJob *)this);}
+
+          XrdCmsDrop(int nid, int inst) : XrdJob("drop node"), nodeP(0),
+                                          nodeEnt(nid), nodeInst(inst)
+                    {Sched->Schedule((XrdJob *)this, time(0)+Config.DRPDelay);}
+
          ~XrdCmsDrop() {}
 
-int  nodeEnt;
-int  nodeInst;
+XrdCmsNode *nodeP;
+int         nodeEnt;
+int         nodeInst;
 };
   
 /******************************************************************************/
@@ -205,14 +211,14 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
                    sendAList(NodeTab[Slot]->Link);
 
                 DEBUG(lp->ID << " bumps " << NodeTab[Slot]->Ident <<" #" <<Slot);
-                NodeTab[Slot]->Lock();
+                NodeTab[Slot]->Lock(true);
                 Remove("redirected", NodeTab[Slot], -1);
                 act = "Shoved ";
                }
        NodeTab[Slot] = nP = new XrdCmsNode(lp, theIF, theNID, port, 0, Slot);
        if (!cidP) cidP = XrdCmsClustID::AddID(theNID);
        if ((cidP->AddNode(nP, SpecAlt))) nP->cidP = cidP;
-          else {delete nP; NodeTab[Slot] = 0; return 0;}
+          else {delete nP; NodeTab[Slot] = 0; return 0;} // OK to do delete!
       }
 
 // Indicate whether this snode can be redirected
@@ -271,6 +277,8 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
 /* Private:                       A d d A l t                                 */
 /******************************************************************************/
   
+// Warning STMutex must be held by the caller!
+
 XrdCmsNode *XrdCmsCluster::AddAlt(XrdCmsClustID *cidP, XrdLink *lp,
                                   int port, int Status, int sport,
                                   const char *theNID, const char *theIF)
@@ -291,7 +299,7 @@ XrdCmsNode *XrdCmsCluster::AddAlt(XrdCmsClustID *cidP, XrdLink *lp,
 //
    if (cidP->Avail())
       {nP = new XrdCmsNode(lp, theIF, theNID, port, 0, slot);
-       if (!(cidP->AddNode(nP, true))) {delete nP; nP = 0;}
+       if (!(cidP->AddNode(nP, true))) {delete nP; nP = 0;} // OK to do delete!
       }
 
 // Check if we were actually able to add this node
@@ -308,7 +316,7 @@ XrdCmsNode *XrdCmsCluster::AddAlt(XrdCmsClustID *cidP, XrdLink *lp,
       {setAltMan(nP->NodeID, nP->Link, sport);
        Say.Emsg("AddAlt", nP->Ident, "replacing dropped", pP->Ident);
        NodeTab[slot] = nP;
-       delete pP;
+       pP->DropJob = new XrdCmsDrop(pP); // Schedule deletion
       }
 
 // Hook the node to the cluster table and return
@@ -340,7 +348,7 @@ void XrdCmsCluster::BlackList(XrdOucTList *blP)
            {inBL = (blP && (blRD = XrdCmsBlackList::Present(nP->Name(), blP)));
             if ((!inBL && !(nP->isBad & XrdCmsNode::isBlisted))
             ||  ( inBL &&  (nP->isBad & XrdCmsNode::isBlisted))) continue;
-            nP->Lock();
+            nP->Lock(true);
             STMutex.UnLock();
             if (inBL)
                {nP->isBad |= XrdCmsNode::isBlisted | XrdCmsNode::isDoomed;
@@ -384,7 +392,7 @@ SMask_t XrdCmsCluster::Broadcast(SMask_t smask, const struct iovec *iod,
 //
    for (i = 0; i <= STHi; i++)
        {if ((nP = NodeTab[i]) && nP->isNode(bmask))
-           {nP->Lock();
+           {nP->Lock(true);
             STMutex.UnLock();
             if (nP->Send(iod, iovcnt, iotot) < 0) 
                {unQueried |= nP->Mask();
@@ -462,7 +470,7 @@ int XrdCmsCluster::Broadsend(SMask_t Who, XrdCms::CmsRRHdr &Hdr,
 //
 do{for (i = Beg; i <= Fin; i++)
        {if ((nP = NodeTab[i]) && nP->isNode(Who))
-           {nP->Lock();
+           {nP->Lock(true);
             STMutex.UnLock();
             if (nP->Send(ioV, 2, ioTot) >= 0) {nP->UnLock(); return 1;}
             DEBUG(nP->Ident <<" is unreachable");
@@ -567,7 +575,6 @@ XrdCmsSelected *XrdCmsCluster::List(SMask_t mask, CmsLSOpts opts, bool &oksel)
                sip->Status |= XrdCmsSelected::Suspend;
             if (nP->isRW     ) sip->Status |= XrdCmsSelected::isRW;
             if (nP->isMan    ) sip->Status |= XrdCmsSelected::isMangr;
-//???       nP->UnLock();
             sipp = sip;
            }
    STMutex.UnLock();
@@ -719,7 +726,7 @@ void *XrdCmsCluster::MonRefs()
            {for (i = 0; i <= STHi; i++)
                 if ((nP = NodeTab[i])
                 &&  (resetWR || (doReset && nP->isNode(resetMask))) )
-                    {nP->Lock();
+                    {nP->Lock(true);
                      if (resetW || doReset) nP->RefW=0;
                      if (resetR || doReset) nP->RefR=0;
                      nP->Shrem = nP->Share;
@@ -741,6 +748,14 @@ void *XrdCmsCluster::MonRefs()
 /*                                R e m o v e                                 */
 /******************************************************************************/
 
+// Warning! The node object must be locked upon entry. The lock is released
+//          upon deletion of the object.
+
+void XrdCmsCluster::Remove(XrdCmsNode *theNode)
+{
+     theNode->DropJob  = new XrdCmsDrop(theNode);
+}
+
 // Warning! The node object must be locked upon entry. The lock is released
 //          prior to returning to the caller. This entry obtains the node
 //          table lock. When immed != 0 then the node is immediately dropped.
@@ -766,17 +781,17 @@ void XrdCmsCluster::Remove(const char *reason, XrdCmsNode *theNode, int immed)
                                 myNID = node->ID(myInst);
                                 if (!hasLK)
                                    {myNode->UnLock();
-                                    myMutex->Lock();
-                                    myNode->Lock();
+                                    myMutex->Lock(); // Get global lock
+                                    myNode->Lock(true);
                                    }
                                }
                       ~theLocks()
                                {if (myNode)
                                    {if (doDrop)
-                                       {myNode->DropTime = 0;
-                                        myNode->DropJob  = 0;
-                                        myNode->isBound  = 0;
-                                        myNode->UnLock(); delete myNode;
+                                       {myNode->isBound  = 0;
+                                        myNode->DropTime = 0;
+                                        myNode->DropJob  = new XrdCmsDrop(myNode);
+                                        myNode->UnLock();
                                        } else myNode->UnLock();
                                    }
                                 if (!hasLK) myMutex->UnLock();
@@ -928,10 +943,21 @@ int XrdCmsCluster::Select(XrdCmsSelect &Sel)
       }
 
 // If we are running a shared file system preform an optional restricted
-// pre-selection and then do a standard selection.
+// pre-selection and then do a standard selection. Since all nodes are equal,
+// make sure the client is needlessly avoiding them as this signals an error.
 //
    if (baseFS.isDFS())
-      {pmask = amask;
+      {if (Sel.nmask && !(Sel.Opts & XrdCmsSelect::NoTryLim))
+          {pmask = (isRW ? pinfo.rwvec : pinfo.rovec) & Sel.nmask;
+           if (!(Sel.Opts & XrdCmsSelect::Online))
+              pmask |= pinfo.ssvec & Sel.nmask;
+           if (pmask && maxBits(pmask, baseFS.dfsTries()))
+              {Sel.Resp.DLen = snprintf(Sel.Resp.Data, sizeof(Sel.Resp.Data)-1,
+               "Too many attempts to gain dfs %s access to the file", Amode)+1;
+               return RetryErr;
+              }
+          }
+       pmask = amask;
        smask = (Sel.Opts & XrdCmsSelect::Online ? 0 : pinfo.ssvec & amask);
        if (baseFS.Trim())
           {Sel.Resp.DLen = 0;
@@ -961,9 +987,13 @@ int XrdCmsCluster::Select(XrdCmsSelect &Sel)
            else if (Sel.Vec.bf) pmask = smask = 0;
            else if (Sel.Vec.hf)
                    {if (Sel.Opts & XrdCmsSelect::NewFile) return SelFail(Sel,eExists);
-                    if (!(Sel.Opts & XrdCmsSelect::isMeta) && Config.DoMWChk
-                    &&  Multiple(Sel.Vec.hf))             return SelFail(Sel,eDups);
-                    if (!(pmask = Sel.Vec.hf & amask))    return SelFail(Sel,eROfs);
+                    if (!(Sel.Opts & XrdCmsSelect::MWFiles))
+                       {if (!(Sel.Opts & XrdCmsSelect::isMeta)
+                        &&  maxBits(Sel.Vec.hf,2))      return SelFail(Sel,eDups);
+                        if ((Sel.Vec.hf & pinfo.rwvec)
+                        !=  (Sel.Vec.hf & pinfo.rovec)) return SelFail(Sel,eROfs);
+                       }
+                    if (!(pmask = Sel.Vec.hf & amask))  return SelFail(Sel,eNoSel);
                     smask = 0;
                    }
            else if (Sel.Opts & (XrdCmsSelect::Trunc | XrdCmsSelect::NewFile))
@@ -1014,9 +1044,25 @@ int XrdCmsCluster::Select(XrdCmsSelect &Sel)
 //
    if (noSel) return 0;
 
-// Select a node
+// Check if we have no useable servers
 //
    if (dowt) return Unuseable(Sel);
+
+// Check if should eliminate staging servers. We may need to do this if the
+// client has been eliminating too many of them as they all should be equal.
+//
+   if (Sel.nmask && pinfo.ssvec && !(Sel.Opts & XrdCmsSelect::NoTryLim)
+   &&  maxBits(Sel.nmask & pinfo.ssvec, baseFS.stgTries()))
+      {if (!pmask)
+          {Sel.Resp.DLen = snprintf(Sel.Resp.Data, sizeof(Sel.Resp.Data)-1,
+           "Too many attempts to stage %s access to the file", Amode)+1;
+           return RetryErr;
+          }
+       smask = 0;
+      }
+
+// Select a node
+//
    return SelNode(Sel, pmask, smask);
 }
 
@@ -1071,8 +1117,8 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
        if (nP)
           {if (isrw)
               if (nP->isNoStage || nP->DiskFree < nP->DiskMinF)    nP = 0;
-                 else {SelWcnt++; nP->RefTotW++; nP->RefW++; nP->Lock();}
-              else    {SelRcnt++; nP->RefTotR++; nP->RefR++; nP->Lock();}
+                 else {SelWcnt++; nP->RefTotW++; nP->RefW++; nP->Lock(true);}
+              else    {SelRcnt++; nP->RefTotR++; nP->RefR++; nP->Lock(true);}
           }
       }
    STMutex.UnLock();
@@ -1106,6 +1152,10 @@ int XrdCmsCluster::SelFail(XrdCmsSelect &Sel, int rc)
                   break;
     case eNoRep:  etext = "Unable to replicate file; no new sites available.";
                   break;
+    case eNoSel:  etext = (Sel.Vec.hf & Sel.nmask
+                        ? "Unable to write file; eligible servers shunned."
+                        : "Unable to write file; r/w exports not found.");
+                  break;
     default:      etext = "Unable to access file; file does not exist.";
                   break;
    };
@@ -1120,9 +1170,10 @@ int XrdCmsCluster::SelFail(XrdCmsSelect &Sel, int rc)
   
 void XrdCmsCluster::Space(SpaceData &sData, SMask_t smask)
 {
-   int i;
    XrdCmsNode *nP;
    SMask_t bmask;
+   int i;
+   bool doAll = !baseFS.isDFS();
 
 // Obtain a lock on the table and screen out peer nodes
 //
@@ -1132,12 +1183,16 @@ void XrdCmsCluster::Space(SpaceData &sData, SMask_t smask)
 // Run through the table getting space information
 //
    for (i = 0; i <= STHi; i++)
-       if ((nP = NodeTab[i]) && nP->isNode(bmask)
-       && !(nP->isOffline)   && nP->isRW)
-          {sData.Total += nP->DiskTotal;
-           sData.sNum++;
-           if (sData.sFree < nP->DiskFree)
-              {sData.sFree = nP->DiskFree; sData.sUtil = nP->DiskUtil;}
+       if ((nP = NodeTab[i]) && nP->isNode(bmask) && !(nP->isOffline))
+          {if (doAll || !sData.Total) 
+              {sData.Total += nP->DiskTotal;
+               sData.TotFr += nP->DiskFree;
+              }
+           if (nP->isRW & XrdCmsNode::allowsSS)
+              {sData.sNum++;
+               if (sData.sFree < nP->DiskFree)
+                  {sData.sFree = nP->DiskFree; sData.sUtil = nP->DiskUtil;}
+              }
            if (nP->isRW & XrdCmsNode::allowsRW)
               {sData.wNum++;
                if (sData.wFree < nP->DiskFree)
@@ -1317,10 +1372,11 @@ XrdCmsNode *XrdCmsCluster::calcDelay(XrdCmsSelector &selR)
 /*                                  D r o p                                   */
 /******************************************************************************/
   
-// Warning: STMutex must be locked upon entry; the caller must release it.
-//          This method may only be called via Remove() either directly or via
-//          a defered job scheduled by that method. This method actually
-//          deletes the node object.
+// Warning: STMutex must be locked upon entry and the caller must release it
+//          if this method is called directily. Otherwise, the mutex will be
+//          obtained and released. Also, this method may only be called via
+//          Remove() either directly or via a defered job scheduled by that
+//          method. This method actually deletes the node object.
 
 int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
 {
@@ -1328,10 +1384,15 @@ int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
    XrdCmsNode *nP;
    char hname[512];
 
+// If we are being called outside of a scheduled job, obtain the mutex
+//
+   if (djp) STMutex.Lock();
+
 // Make sure this node is the right one
 //
    if (!(nP = NodeTab[sent]) || nP->Inst() != sinst)
       {if (nP && djp == nP->DropJob) {nP->DropJob = 0; nP->DropTime = 0;}
+       if (djp) STMutex.UnLock();
        DEBUG(sent <<'.' <<sinst <<" cancelled.");
        return 0;
       }
@@ -1340,6 +1401,7 @@ int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
 //
    if (djp && time(0) < nP->DropTime)
       {Sched->Schedule((XrdJob *)djp, nP->DropTime);
+       if (djp) STMutex.UnLock();
        return 1;
       }
 
@@ -1380,13 +1442,16 @@ int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
 //
    if (nP->NodeMask) Cache.Drop(nP->NodeMask, sent, STHi);
 
+// We can now delete the node object if we were called via a job as we are on
+// a different thread. Direct calls require that we schedule the deletion as
+// it may take a long time if there are oustanding references to this node.
+//
+   if (djp) {STMutex.UnLock(); nP->Delete(STMutex);}
+      else nP->DropJob = new XrdCmsDrop(nP);
+
 // Document the drop
 //
    Say.Emsg("Drop_Node", hname, "dropped.");
-
-// Delete the node object
-//
-   delete nP;
    return 0;
 }
 
@@ -1422,6 +1487,27 @@ int XrdCmsCluster::Multiple(SMask_t mVec)
    return isMult[mVec];
 }
   
+/******************************************************************************/
+/*                               m a x B i t s                                */
+/******************************************************************************/
+  
+bool XrdCmsCluster::maxBits(SMask_t mVec, int mbits)
+{
+   int count = 0;
+
+// Count bits. This is the fastest way assuming few bits are set
+//
+   while(mVec)
+        {mVec &= (mVec - 1);
+         count++;
+         if (count >= mbits) return true;
+        }
+
+// Indicate we have not reached the maximum bits set
+//
+   return false;
+}
+
 /******************************************************************************/
 /*                                R e c o r d                                 */
 /******************************************************************************/
@@ -1606,7 +1692,7 @@ XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
 // Check for overloaded node and return result
 //
    if (!sp) return calcDelay(selR);
-   sp->Lock();
+   sp->Lock(true);
    RefCount(sp, Multi, selR.needSpace);
    return sp;
 }
@@ -1650,7 +1736,7 @@ XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
 // Check for overloaded node and return result
 //
    if (!sp) return calcDelay(selR);
-   sp->Lock();
+   sp->Lock(true);
    RefCount(sp, Multi, selR.needSpace);
    return sp;
 }
@@ -1687,7 +1773,7 @@ XrdCmsNode *XrdCmsCluster::SelbyRef(SMask_t mask, XrdCmsSelector &selR)
 // Check for overloaded node and return result
 //
    if (!sp) return calcDelay(selR);
-   sp->Lock();
+   sp->Lock(true);
    RefCount(sp, Multi, selR.needSpace);
    return sp;
 }
