@@ -172,7 +172,7 @@ public:
 
 static void Start() {static PingClock selfie;}
 
-          PingClock() : XrdJob("ping clock") {DoIt();}
+          PingClock() : XrdJob(".ping clock") {DoIt();}
          ~PingClock() {}
 private:
 };
@@ -406,7 +406,7 @@ int XrdCmsConfig::Configure2()
 
 // If we need a name library, load it now
 //
-   if ((LocalRoot || RemotRoot) && ConfigN2N()) NoGo = 1;
+   if ((LocalRoot || RemotRoot || N2N_Lib) && ConfigN2N()) NoGo = 1;
 
 // Configure the OSS, the base filesystem, and initialize the prep queue
 //
@@ -483,6 +483,7 @@ int XrdCmsConfig::ConfigXeq(char *var, XrdOucStream &CFile, XrdSysError *eDest)
    TS_Xeq("allow",         xallow);  // Manager, non-dynamic
    TS_Xeq("altds",         xaltds);  // Server,  non-dynamic
    TS_Xeq("blacklist",     xblk);    // Manager, non-dynamic
+   TS_Xeq("cidtag",        xcid);    // Any,     non-dynamic
    TS_Xeq("defaults",      xdefs);   // Server,  non-dynamic
    TS_Xeq("dfs",           xdfs);    // Any,     non-dynamic
    TS_Xeq("export",        xexpo);   // Any,     non-dynamic
@@ -636,6 +637,7 @@ void XrdCmsConfig::ConfigDefaults(void)
    QryDelay =-1;
    QryMinum = 0;
    LUPHold  = 178;
+   DELDelay = 960;  // 15 minutes
    DRPDelay = 10*60;
    PSDelay  = 0;
    RWDelay  = 2;
@@ -695,6 +697,7 @@ void XrdCmsConfig::ConfigDefaults(void)
    SanList   =0;
    mySID    = 0;
    mySite   = 0;
+   cidTag   = 0;
    ifList    =0;
    perfint  = 3*60;
    perfpgm  = 0;
@@ -746,7 +749,7 @@ int XrdCmsConfig::ConfigN2N()
 
 // Get the plugin
 //
-   if (!(xeq_N2N = n2nLoader.Load(N2N_Lib, *myVInfo))) return 1;
+   if (!(xeq_N2N = n2nLoader.Load(N2N_Lib, *myVInfo, &theEnv))) return 1;
 
 // Optimize the local case
 //
@@ -1158,7 +1161,7 @@ char *XrdCmsConfig::setupSid()
 
 // Generate the system ID and set the cluster ID
 //
-   return XrdCmsSecurity::setSystemID(tp, myInsName, myName, sfx);
+   return XrdCmsSecurity::setSystemID(tp, myInsName, myName, cidTag, sfx);
 }
 
 /******************************************************************************/
@@ -1382,6 +1385,40 @@ int XrdCmsConfig::xblk(XrdSysError *eDest, XrdOucStream &CFile, bool iswl)
    return 0;
 }
   
+/******************************************************************************/
+/*                                  x c i d                                   */
+/******************************************************************************/
+
+/* Function: xcid
+
+   Purpose:  To parse the directive: cidtag <tag>
+
+             <tag>     a 1- to 16-character cluster ID tag.
+
+  Output: 0 upon success or !0 upon failure.
+*/
+
+int XrdCmsConfig::xcid(XrdSysError *eDest, XrdOucStream &CFile)
+{
+    char *val;
+
+// Get the path
+//
+   if (!(val = CFile.GetWord()) || !val[0])
+      {eDest->Emsg("Config", "tag not specified"); return 1;}
+
+// Make sure it is not too long
+//
+   if ((int)strlen(val) > 16)
+      {eDest->Emsg("Config", "tag is > 16 characters"); return 1;}
+
+// Record the tag
+//
+   if (cidTag) free(cidTag);
+   cidTag = strdup(val);
+   return 0;
+}
+  
 /******************************************************************************/
 /*                                x d e l a y                                 */
 /******************************************************************************/
@@ -1394,13 +1431,16 @@ int XrdCmsConfig::xblk(XrdSysError *eDest, XrdOucStream &CFile, bool iswl)
                                            [suspend <sec>] [drop <sec>]
                                            [service <sec>] [hold <msec>]
                                            [peer <sec>] [rw <lvl>] [qdl <sec>]
-                                           [qdn <cnt>]
+                                           [qdn <cnt>] [delnode <sec>]
+                                           [nostage <cnt>]
 
+   delnode   <sec>     maximum seconds to wait to be able to delete a node.
    discard   <cnt>     maximum number a message may be forwarded.
    drop      <sec>     seconds to delay a drop of an offline server.
    full      <sec>     seconds to delay client when no servers have space.
    hold      <msec>    millseconds to optimistically hold requests.
    lookup    <sec>     seconds to delay client when finding a file.
+   nostage   <cnt>     Maximum number of staging reselections allowed.
    overload  <sec>     seconds to delay client when all servers overloaded.
    peer      <sec>     maximum seconds client may be delayed before peer
                        selection is triggered.
@@ -1422,15 +1462,17 @@ int XrdCmsConfig::xblk(XrdSysError *eDest, XrdOucStream &CFile, bool iswl)
 int XrdCmsConfig::xdelay(XrdSysError *eDest, XrdOucStream &CFile)
 {   char *val;
     const char *etxt = "invalid delay option";
-    int  i, ppp, minV = 1, ispercent = 0;
+    int  i, ppp, minV = 1, ispercent = 0, noStage = 0;
     static struct delayopts {const char *opname; int *oploc; int istime;}
            dyopts[] =
        {
+        {"delnode",  &DELDelay, 1},
         {"discard",  &MsgTTL,   0},
         {"drop",     &DRPDelay, 1},
         {"full",     &DiskWT,  -1},
         {"hold",     &LUPHold,  0},
         {"lookup",   &LUPDelay, 1},
+        {"nostage",  &noStage,  01},
         {"overload", &MaxDelay,-1},
         {"peer",     &PSDelay,  1},
         {"qdl",      &QryDelay, 1},
@@ -1481,7 +1523,11 @@ int XrdCmsConfig::xdelay(XrdSysError *eDest, XrdOucStream &CFile)
               eDest->Say("Config warning: ignoring invalid delay option '",val,"'.");
            val = CFile.GetWord();
           }
-     return 0;
+
+// Set the nostage option here
+//
+   if (noStage) baseFS.SetTries(false, noStage);
+   return 0;
 }
 
 /******************************************************************************/
@@ -1540,6 +1586,8 @@ int XrdCmsConfig::xdefs(XrdSysError *eDest, XrdOucStream &CFile)
                                  redirecting a client. This is the
                                  default for non-proxy configurations.    top
 
+             retries <n>         Maximum number of select retries.
+
    Type: Any, non-dynamic.
 
    Output: 0 upon success or !0 upon failure.
@@ -1549,7 +1597,7 @@ int XrdCmsConfig::xdfs(XrdSysError *eDest, XrdOucStream &CFile)
 {
     int Opts = XrdCmsBaseFS::DFSys | (isProxy ? XrdCmsBaseFS::Immed : 0)
              | (!isManager && isServer ? XrdCmsBaseFS::Servr: 0);
-    int Hold = 0, limCent = 0, limFix = 0, limV = 0, qMax = 0;
+    int Hold = 0, limCent = 0, limFix = 0, limV = 0, qMax = 0, rTry = 0;
     char *val;
 
 // If we are a meta-manager or a peer, ignore this option
@@ -1599,6 +1647,11 @@ do{     if (!strcmp("mdhold",  val))
                   return 1;
                  }
            }
+   else if (!strcmp("retries", val))
+           {if (!(val = CFile.GetWord()))
+               {eDest->Emsg("Config","retries value not specified.");    return 1;}
+            if (XrdOuca2x::a2i(*eDest, "retries value", val, &rTry, 1))  return 1;
+           }
    else {eDest->Emsg("Config", "invalid dfs option '",val,"'."); return 1;}
   } while((val = CFile.GetWord()));
 
@@ -1624,6 +1677,7 @@ do{     if (!strcmp("mdhold",  val))
 
 // All done, simply set the values
 //
+   baseFS.SetTries(true, rTry);
    baseFS.Limit(limV, qMax);
    baseFS.Init(Opts, Hold, Hold*10);
    return 0;
@@ -2618,6 +2672,7 @@ int XrdCmsConfig::xspace(XrdSysError *eDest, XrdOucStream &CFile)
     char *val;
     int i, alinger = -1, arecalc = -1, minfP = -1, hwmP = -1;
     long long minf = -1, hwm = -1;
+    bool haveopt = false;
 
     while((val = CFile.GetWord()))
       {    if (!strcmp("linger", val))
@@ -2635,7 +2690,7 @@ int XrdCmsConfig::xspace(XrdSysError *eDest, XrdOucStream &CFile)
                   {eDest->Emsg("Config", "space min value not specified"); return 1;}
                break;
               }
-      else if (!strcmp("mwfiles", val)) DoMWChk = 0;
+      else if (!strcmp("mwfiles", val)) {DoMWChk = 0; haveopt = true;}
       else if (isdigit(*val)) break;
       else {eDest->Emsg("Config", "invalid space parameters"); return 1;}
       }
@@ -2682,7 +2737,7 @@ int XrdCmsConfig::xspace(XrdSysError *eDest, XrdOucStream &CFile)
 
     if (val) {eDest->Emsg("Config", "invalid space parameter -", val); return 1;}
     
-    if (alinger < 0 && arecalc < 0 && minf < 0)
+    if (!haveopt && alinger < 0 && arecalc < 0 && minf < 0)
        {eDest->Emsg("Config", "no space values specified"); return 1;}
 
     if (alinger >= 0) DiskLinger = alinger;
