@@ -31,7 +31,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-#include <netinet/in.h>
 #include <time.h>
 #include <pwd.h>
 #include <sys/types.h>
@@ -144,7 +143,7 @@ volatile void *XrdSutMemSet(volatile void *dst, int c, int len)
    // (see discussion there)
    volatile char *buf;
 
-   for (buf = (volatile char *)dst; len; buf[--len] = c);
+   for (buf = (volatile char *)dst; len; buf[--len] = c) {}
    return dst;
 }
 
@@ -616,13 +615,12 @@ XrdSutFileLocker::XrdSutFileLocker(int fd, ELockType lock)
    fdesk = fd;
 
    // Exclusive lock of the whole file
-   short lockmode = (lock == XrdSutFileLocker::kExcl) ? (F_WRLCK | F_RDLCK)
-                                                      :  F_RDLCK;
-#ifdef __macos__
-   struct flock flck = {0, 0, 0, lockmode, SEEK_SET};
-#else
-   struct flock flck = {lockmode, SEEK_SET, 0, 0};
-#endif
+   int lockmode = (lock == XrdSutFileLocker::kExcl) ? (F_WRLCK | F_RDLCK)
+                                                    :  F_RDLCK;
+   struct flock flck;
+   memset(&flck, 0, sizeof(flck));
+   flck.l_type   = lockmode;
+   flck.l_whence = SEEK_SET;
    if (fcntl(fdesk, F_SETLK, &flck) != 0)
       // Failure
       return;
@@ -639,11 +637,10 @@ XrdSutFileLocker::~XrdSutFileLocker()
       return;
    //
    // Unlock the file
-#ifdef __macos__
-   struct flock flck = {0, 0, 0, F_UNLCK, SEEK_SET};
-#else
-   struct flock flck = {F_UNLCK, SEEK_SET, 0, 0};
-#endif
+   struct flock flck = {F_UNLCK, SEEK_SET, 0, 0, 0};
+   memset(&flck, 0, sizeof(flck));
+   flck.l_type   = F_UNLCK;
+   flck.l_whence = SEEK_SET;
    fcntl(fdesk, F_SETLK, &flck);
 }
 
