@@ -212,7 +212,8 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 // transfer size to the buffer size (before it was a reasonable 256K).
 //
    if (!(as_miniosz = as_segsize/2)) as_miniosz = as_segsize;
-   maxTransz = maxBuffsz = BPool->MaxSize();
+   n = (pi->theEnv ? pi->theEnv->GetInt("MaxBuffSize") : 0);
+   maxTransz = maxBuffsz = (n ? n : BPool->MaxSize());
    memset(Route, 0, sizeof(Route));
 
 // Now process and configuration parameters
@@ -774,7 +775,7 @@ int XrdXrootdProtocol::xdig(XrdOucStream &Config)
 
 /* Function: xexp
 
-   Purpose:  To parse the directive: export <path> [lock|nolock]
+   Purpose:  To parse the directive: export <path> [lock|nolock] [mwfiles]
 
              <path>    the path to be exported.
 
@@ -795,9 +796,11 @@ int XrdXrootdProtocol::xexp(XrdOucStream &Config)
 
 // Get export lock option
 //
-   if ((val = Config.GetWord()))
-      {if (!strcmp("nolock", val)) popt = XROOTDXP_NOLK;
-          else if (strcmp("lock", val)) Config.RetToken();
+   while((val = Config.GetWord()))
+      {     if (!strcmp( "nolock", val)) popt |=  XROOTDXP_NOLK;
+       else if (!strcmp(   "lock", val)) popt &= ~XROOTDXP_NOLK;
+       else if (!strcmp("mwfiles", val)) popt |=  XROOTDXP_NOMWCHK;
+       else {Config.RetToken(); break;}
       }
 
 // Add path to configuration
@@ -810,6 +813,7 @@ int XrdXrootdProtocol::xexp(XrdOucStream &Config)
 int XrdXrootdProtocol::xexpdo(char *path, int popt)
 {
    char *opaque;
+   int   xopt;
 
 // Check if we are exporting a generic name
 //
@@ -830,7 +834,8 @@ int XrdXrootdProtocol::xexpdo(char *path, int popt)
 
 // Record the path
 //
-   if (!Squash(path)) XPList.Insert(path, popt);
+   if (!(xopt = Squash(path)) || xopt != (popt|XROOTDXP_OK))
+      XPList.Insert(path, popt);
    return 0;
 }
   
